/**
 * W-SLDA Toolkit
 * 
 * This file provides template for processing of wave-functions generated by st-wslda-2d.
 * 
 * Copy this file to your project folder and compile using:
 *    gcc -std=gnu99 st-processwf-2d-template.c -I. -I$WSLDA/hpc-engine -o st-processwf-2d-template -lm -lfftw3
 * 
 * */ 

// Standard libraries
#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <complex.h>

// W-SLDA Toolkit API
#include "wslda_toolkit.h"


int main( int argc , char ** argv ) 
{ 
    // command line
    if(argc!=2)
    {
        printf("Usage: %s prefix\n", argv[0]);
        return( EXIT_FAILURE ) ;        
    }
    
    char *prefix = argv[1];
    printf("# PREFIX FOR FILES: %s\n", prefix);
    
    // lattice settings
    int nx, ny, nz;
    double dx, dy, dz;
    
    // other variables
    int ix, iy, iz, ixyz; // lattice iterators
    int ierr;             // error flag
    char file_name[256];
    double kF, ec, beta;
    double mu[2];
    int nwf;
    
    sprintf(file_name, "%s/s2dpca.info", prefix);
    printf("# READING INFO FILE: %s\n", file_name);
    file_operationl(read_checkpoint_info_pca(file_name, &nwf, &nx, &ny, &nz, &dx, &dy, &dz, &kF, mu, &ec, &beta));

    printf("# LATTICE: %d x %d x %d\n", nx, ny, nz);
    printf("# SPACING: %.2f x %.2f x %.2f\n", dx, dy, dz);
    printf("# VOLUME : %.2f x %.2f x %.2f\n", dx * nx, dy * ny, dz * nz);
    
    // states information for each kz
    int ikz, iwf;
    double ei, kz;
    double complex *u, *v;
    cppmallocl(u, nx * ny, double complex);
    cppmallocl(v, nx * ny, double complex);
    
    // file pointers 
    FILE *pFile_en, *pFile_kkz, *pFile_wfu, *pFile_wfv;
    
    for (ikz = 0; ikz < nz / 2; ikz++)
    {
        // read file header
        sprintf(file_name, "%s/s2dpca.%04d.info", prefix, ikz);
        printf("# OPENING: %s\n", file_name);
        file_operationl(read_checkpoint_info_pca(file_name, &nwf, &nx, &ny, &nz, &dx, &dy, &dz, &kF, mu, &ec, &beta));
        
        // open files
        sprintf(file_name, "%s/s2dpca.%04d.en", prefix, ikz);
        pFile_en = fopen(file_name, "rb");
        if(pFile_en==NULL) {printf("ERROR: Cannot open %s\n", file_name); return( EXIT_FAILURE ) ;}
        sprintf(file_name, "%s/s2dpca.%04d.kkz", prefix, ikz);
        pFile_kkz = fopen(file_name, "rb");
        if(pFile_en==NULL) {printf("ERROR: Cannot open %s\n", file_name); return( EXIT_FAILURE ) ;}
        sprintf(file_name, "%s/s2dpca.%04d.wfu", prefix, ikz);
        pFile_wfu = fopen(file_name, "rb");
        if(pFile_en==NULL) {printf("ERROR: Cannot open %s\n", file_name); return( EXIT_FAILURE ) ;}
        sprintf(file_name, "%s/s2dpca.%04d.wfv", prefix, ikz);
        pFile_wfv = fopen(file_name, "rb");
        if(pFile_en==NULL) {printf("ERROR: Cannot open %s\n", file_name); return( EXIT_FAILURE ) ;}
        
        // read wave-functions
        printf("# PROCESSING WF[%d] FOR ikz=%d\n", nwf, ikz);
        for (iwf = 0; iwf < nwf; iwf++)
        {
            fread(&ei, sizeof(double), 1, pFile_en);  // eigen energy
            fread(&kz, sizeof(double), 1, pFile_kkz); // kz value
            fread(u, sizeof(double complex) * nx * ny, 1, pFile_wfu); // u-component
            fread(v, sizeof(double complex) * nx * ny, 1, pFile_wfv); // v-compoment
            
            // normalize
            ixyz = 0;
            for (ix = 0; ix < nx; ix++) for (iy = 0; iy < ny; iy++)
            {
                u[ixyz] /= sqrt(dx * dy);
                v[ixyz] /= sqrt(dx * dy);
                ixyz++;
            }
            
            // ...
            // ... WRITE HERE YOUR CODE ...
            // ...
            
        }
        
        
        // close files
        fclose(pFile_en);
        fclose(pFile_kkz);
        fclose(pFile_wfu);
        fclose(pFile_wfv);
        
    }
    
    /* Arrays will be cleared automatically */
    return( EXIT_SUCCESS ) ;
}
