/**
 * W-SLDA Toolkit
 * Author: Gabriel Wlazlowski
 * Creation date: 15-10-2020
 * 
 * Declaration file for errors reporting
 * */ 


#include "wslda_errors.h"

void wprintf( const char * format, ... );
void wfprintf(FILE *stream,  const char * format, ... );

#include "jdb.h"
extern int wsldapid; // process id - global variable
void something_to_cheer_you_up(FILE *stream)
{
    wfprintf(stream, "==========================================================================\n");
    wfprintf(stream, "Upset with the error? Here is something to cheer you up:\n");
    jdb_message(stream);
    wfprintf(stream, "==========================================================================\n");
}
void something_to_cheer_you_up_pid0(FILE *stream)
{
    if(wsldapid==0) something_to_cheer_you_up(stream);
}

void report_error(int errcode, FILE *stream)
{
    if(errcode==WSLDA_OK) return; // no reporting
    
    wfprintf(stream, "==========================================================================\n");
    wfprintf(stream, "WSLDA ERROR DESCRIPTION:\n");
    switch(errcode)
    {
        case WSLDA_ERR_CANNOT_CREATE_DIR:
            wfprintf(stream, "\tCannot create directory for binary files.\n");
            wfprintf(stream, "\tCheck your permissions.\n");
            break;
            
        case WSLDA_ERR_CANNOT_OPEN_FILE:
            wfprintf(stream, "\tCannot open binary file.\n");
            wfprintf(stream, "\tCheck if `inprefix` tag is correctly set in input file.\n");
            break;
            
        case WSLDA_ERR_CANNOT_OVERWRITE:
            wfprintf(stream, "\tAttempt of overwriting existing file has taken.\n");
            wfprintf(stream, "\tInput file tag overwrite=0 does not allow for this.\n");
            wfprintf(stream, "\tChange overwrite tag or outprefix tag in input file.\n");
            break;
            
        case WSLDA_ERR_BINARY_FILE_CORRUPTED:
            wfprintf(stream, "\tInput binary files do not satisfy expected sum-rules.\n");
            wfprintf(stream, "\tThey seems to be damaged.\n");
            break;
            
        case WSLDA_ERR_INOCRRECT_PQ:
            wfprintf(stream, "\tParameters p and q in input file are set incorrectly.\n");
            wfprintf(stream, "\tCorrect input file.\n");
            break;
            
        case WSLDA_ERR_S3DPCA_INFO_FILES:
            wfprintf(stream, "\tInput binary files do not satisfy expected sum-rules.\n");
            wfprintf(stream, "\tThis situation may occur if value of iogroups used in static code (st)\n");
            wfprintf(stream, "\t   is different from value of iogroups provided for time-dependent (td) code.\n");
            wfprintf(stream, "\tCorrect iogropus in input file.\n");
            wfprintf(stream, "\tIf iogroups is set correctly this error indicates that binary files\n");
            wfprintf(stream, "\t   with wave-functions from static code may to be damaged.\n");
            break;
            
        case WSLDA_ERR_S3DPCA_INFO_FILES_MISSING_FILE:
            wfprintf(stream, "\tInput binary files do not satisfy expected sum-rules.\n");
            wfprintf(stream, "\tThis situation may occur if value of iogroups is set incorrectly.\n");
            wfprintf(stream, "\tIn general it should have the same value as used in static code (st).\n");
            wfprintf(stream, "\tIn some cases static code (st) cannot generate requested iogroups (problem is too small).\n");
            wfprintf(stream, "\tCheck maximum value of number appearing in file names `inprefix`\\s3dpca.????.info\n");
            wfprintf(stream, "\t  and in input file set iogroups as (maximum value+1).\n");
            wfprintf(stream, "\tIf iogroups is set correctly this error indicates that binary files\n");
            wfprintf(stream, "\t   with wave-functions from static code may to be damaged.\n");
            break;
            
        case WSLDA_ERR_ABDG_NOT_SET:
            wfprintf(stream, "\tFor calculations with FUNCTIONAL==BDG\n");
            wfprintf(stream, "\tit is required to set aBdG in input file!\n");
            break;
            
        case WSLDA_ERR_TD_CANNOT_LOAD_CHECKPOINT_DATA:
            wfprintf(stream, "\tTD code cannot load data from checkpoint files!\n");
            wfprintf(stream, "\tSuggestions that may help to solve the problem:\n");
            wfprintf(stream, "\t\t- inprefix should point to folder with checkpoint files\n");
            wfprintf(stream, "\t\t- you should have read permission to checkpoint files\n");
            wfprintf(stream, "\t\t- make sure you use the same value of MPI_NP_PER_IO_GROUP as you used for writing\n");
            break;
            
        case WSLDA_ERR_TD_CANNOT_LOAD_CHECKPOINT_NWF:
            wfprintf(stream, "\tTD code cannot read header of checkpoint files!\n");
            wfprintf(stream, "\tSuggestions that may help to solve the problem:\n");
            wfprintf(stream, "\t\t- inprefix should point to folder with checkpoint files\n");
            wfprintf(stream, "\t\t- you should have read permission to checkpoint files\n");
            wfprintf(stream, "\t\t- make sure you use the same value of MPI_NP_PER_IO_GROUP as you used for writing\n");
            break;
            
        case WSLDA_ERR_NAN_DETECTED:
            wfprintf(stream, "\tNot a Number (NaN) has been detected!\n");
            wfprintf(stream, "\tCheck settings of the code!\n");
            break;
            
        case WSLDA_ERR_INF_DETECTED:
            wfprintf(stream, "\tInfinite (Inf) has been detected!\n");
            wfprintf(stream, "\tCheck settings of the code!\n");
            break;
            
        case WSLDA_ERR_INTERPOLATION_NOT_IMPLEMENTED:
            wfprintf(stream, "\tUnsupported case of the interpolation!\n");
            wfprintf(stream, "\tSupported cases for 3D -- ALL sizes bigger/smaller then the imput sizes.\n");
            break;
            
        case WSLDA_ERR_CANNOT_CREATE_CHECKPOINT_FILE:
            wfprintf(stream, "\tCannot create the checkpoint file!\n");
            wfprintf(stream, "\tCheck if you have write permission to the target location.\n");
            break;
    
        case WSLDA_ERR_CANNOT_WRITETO_CHECKPOINT_FILE:
            wfprintf(stream, "\tCannot add entry to the checkpoint file!\n");
            wfprintf(stream, "\tCheck if you have disk space in the target location.\n");
            break;

        case WSLDA_ERR_CANNOT_OPEN_CHECKPOINT_FILE:
            wfprintf(stream, "\tCannot open the checkpoint file!\n");
            wfprintf(stream, "\tCheck if you have read permission to the target location.\n");
            break;    
            
        case WSLDA_ERR_CANNOT_READFROM_CHECKPOINT_FILE:
            wfprintf(stream, "\tCannot read entry from the checkpoint file!\n");
            wfprintf(stream, "\tCheck the correctness of the checkpoint file. It may be corrupted.\n");
            break;
            
        case WSLDA_ERR_INCOMPATIBLE_CHECKPOINT_FILE:
            wfprintf(stream, "\tThe checkpoint file is incompatible with the code settings!\n");
            wfprintf(stream, "\tYou cannot use provided checkpoint file to initialize the code!\n");
            break;
            
        case WSLDA_ERR_INTRISTIC_ERROR:
            wfprintf(stream, "\tIt is intrinsic error of W-SLDA Toolkit.\n");
            wfprintf(stream, "\tIt shouldn't have happened, but it did :-(\n");
            wfprintf(stream, "\tPlease report this error to W-SLDA Teams and help us to improve the Toolkit.\n");
            break;
            
        default: 
            wfprintf(stream, "\tThis error does not have description.\n");
    }
    wfprintf(stream, "IF THIS INFORMATION IS NOT SUFFICIENT TO SOLVE YOUR PROBLEM\n");
    wfprintf(stream, "\tCheck wiki pages: https://gitlab.fizyka.pw.edu.pl/wtools/wslda/-/wikis/home\n");
    wfprintf(stream, "\tAsk for help WSLDA developers:\n");
    wfprintf(stream, "\t\tusing Issues reporting system: https://gitlab.fizyka.pw.edu.pl/wtools/wslda/-/issues\n");
    wfprintf(stream, "\t\tor by e-mail: wslda@fizyka.pw.edu.pl\n");
    something_to_cheer_you_up_pid0(stream);
}

void warn_head(FILE *stream)
{
    wfprintf(stream, "# !!! --- WARNING --- WARNING --- WARNING --- WARNING --- WARNING --- WARNING --- !!!\n");
}
void warn_foot(FILE *stream)
{
    wfprintf(stream, "# !!! --- ------- --- ------- --- ------- --- ------- --- ------- --- ------- --- !!!\n");
}

void report_warning(int errcode, FILE *stream)
{
    if(errcode==WSLDA_OK) return; // no reporting
    
    warn_head(stream);

    switch(errcode)
    {
        case WSLDA_WRN_SPINSYMMETRY0:
            wfprintf(stream, "#\t Input file setting: `spinsymmetry 0` not comptible with predefines.h option SPINSYMMETRY_MODE!\n");
            wfprintf(stream, "#\t Check if it is intended!\n");
            wfprintf(stream, "#\t To avoid the code termination forcing: `spinsymmetry 1`!\n");
            break;
            
        case WSLDA_WRN_SPINSYMMETRY1:
            wfprintf(stream, "#\t Input file setting: `spinsymmetry 1` not comptible with commented-out predefines.h option SPINSYMMETRY_MODE!\n");
            wfprintf(stream, "#\t Check if it is intended!\n");
            wfprintf(stream, "#\t To avoid the code termination forcing: `spinsymmetry 0`!\n");
            break;
            
        case WSLDA_WRN_CHECKPOINT_NOT_CONSITENT_BROYDEN:
            wfprintf(stream, "#\t Binary data for Boyden algorithm not consistent with current settings!\n");
            wfprintf(stream, "#\t The data will NOT be loaded!\n");
            break;
            
        case WSLDA_WRN_CHECKPOINT_DOINTERPOLATION:
            wfprintf(stream, "#\t Resolution of the lattice has changed!\n");
            wfprintf(stream, "#\t The code will interpolate given checkpoint data to the new resolution.\n");
            break;
            
        case WSLDA_WRN_CHECKPOINT_DORESIZE:
            wfprintf(stream, "#\t Dimensonality of the lattice has changed!\n");
            wfprintf(stream, "#\t The code will change dimensionality of given checkpoint data to the new lattice.\n");
            break;
            
        case WSLDA_WRN_CHECKPOINT_UNPREDICTED:
            wfprintf(stream, "#\t Checkpoint files is incompatible with code settings!\n");
            wfprintf(stream, "#\t The code will upload the data however the result of this operation may be unpredictable!\n");
            wfprintf(stream, "#\t Make sure you understand what you are doing!\n");
            break;
            
        default: 
            wfprintf(stream, "#\tThis warning does not have description.\n");
    }
    warn_foot(stream);
 
}

void print_warning(int errcode)
{
    report_warning(errcode, stdout); // to output 
    report_warning(errcode, stderr); // and to error process
}
